﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CPM100_Net;
using System.Runtime.InteropServices;

namespace NMT_Protocol
{
    public partial class Form1 : Form
    {
        byte BOARD;
        byte BAUD;
        byte NODE;
        byte STA;

        byte BlockMode = 1;

        string STA_Str = "";
        string EMCY = "";
        string NMTErr = "";

        ushort DelayTime;
        ushort ret;
        string errstr;

        CPM100.NMTErrISR MyNMTErrISR;
        CPM100.EMCYISR MyEMCYISR;

        public Form1()
        {
            InitializeComponent();
        }

        private void UserNMTErrISR()
        {
            byte board, node, mode;
            string tempstr = "";

            if (CPM100.CPM100_GetNMTError(out board, out node, out mode) == 0)
            {
                if (mode == CPM100.CPM_Node_Guarding_Event)
                {
                    tempstr = "NMTErr: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " Guarding Fail";
                }
                else if (mode == CPM100.CPM_Heartbeat_Event)
                {
                    tempstr = "NMTErr: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " Heartbeat Error";
                }
            }

            NMTErr = tempstr;
        }

        private void UserEMCYISR()
        {
            byte board, node;
            byte[] data = new byte[8];
            string tempstr = "";

            if (CPM100.CPM100_GetEMCYData(out board, out node, data) == 0)
            {
                ushort i;

                tempstr = "EMCY: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " :";
                for (i = 0; i < 8; i++)
                {
                    tempstr = tempstr + data[i].ToString("X2") + " ";
                }
            }

            EMCY = tempstr;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            BOARD = 0;
            BAUD = 0;
            NODE = 1;

            STA = 0x01;
            STA_Str = "Operation mode";

            Combo_Board.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Board.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;
            Combo_State.SelectedIndex = 0;

            Btn_Initial.Enabled = true;
            Btn_Shutdown.Enabled = false;
            Btn_AddNode.Enabled = false;
            Btn_AddNode.Visible = true;
            Btn_Remove.Enabled = false;

            pal_NMT.Enabled = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            CPM100.CPM100_ShutdownMaster(BOARD);
        }

        private void Combo_Board_SelectedIndexChanged(object sender, EventArgs e)
        {
            BOARD = (byte)(Combo_Board.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Combo_State_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (Combo_State.SelectedIndex)
            {
                case 0:
                    STA = 0x01;   //Operation mode
                    STA_Str = "Operation mode";
                    break;
                case 1:
                    STA = 0x02;   //Stop mode
                    STA_Str = "Stop mode";
                    break;
                case 2:
                    STA = 0x80;  //Pre-Operation mode
                    STA_Str = "Pre-Operation mode";
                    break;
                case 3:
                    STA = 0x81;  //Reset Node
                    STA_Str = "Reset Node";
                    break;
                case 4:
                    STA = 0x82;  //Reset Communication
                    STA_Str = "Reset Communication";
                    break;
            }
        }

        private void Btn_SetState_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_NMTChangeState(BOARD, NODE, STA, BlockMode);
            if (ret != 0)
            {
                errstr = "Change Slave State Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                errstr = "Change Slave State to" + STA_Str + " OK";
                List_EMCY.Items.Add(errstr);
            }
        }
        
        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_InitMaster(BOARD, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                MyNMTErrISR = new CPM100.NMTErrISR(UserNMTErrISR);
                MyEMCYISR = new CPM100.EMCYISR(UserEMCYISR);

                CPM100.CPM100_InstallNMTErrISR(BOARD, MyNMTErrISR);
                CPM100.CPM100_InstallEMCYISR(BOARD, MyEMCYISR);
                
                Combo_Board.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;

                timer1.Enabled = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_ShutdownMaster(BOARD);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                CPM100.CPM100_RemoveNMTErrISR(BOARD);
                CPM100.CPM100_RemoveEMCYISR(BOARD);

                Combo_Board.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;

                pal_NMT.Enabled = false;

                timer1.Enabled = false;

                List_EMCY.Items.Clear();
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_AddNode(BOARD, NODE, 1, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_Remove.Enabled = true;
                Combo_Node.Enabled = false;

                pal_NMT.Enabled = true;
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_RemoveNode(BOARD, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_Remove.Enabled = false;
                Combo_Node.Enabled = true;

                List_EMCY.Items.Clear();

                pal_NMT.Enabled = false;
            }
        }

        private void Btn_Clear_Click(object sender, EventArgs e)
        {
            List_EMCY.Items.Clear();
        }

        private void Check_Is_Dec(object sender, KeyPressEventArgs e)
        {
            if (!(("1234567890".IndexOf(e.KeyChar) != -1) || ((int)e.KeyChar == 8))) e.Handled = true;
        }

        private void Btn_SetGuard_Click(object sender, EventArgs e)
        {
            ushort GuardTime;
            byte LifeTime;

            GuardTime = Convert.ToUInt16(Text_GuardTime.Text, 10);
            LifeTime = Convert.ToByte(Text_LifeTime.Text);

            ret = CPM100.CPM100_NMTGuarding(BOARD, NODE, GuardTime, LifeTime, BlockMode);
            if (ret != 0)
            {
                errstr = "Set Guarding Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                errstr = "Set Guarding OK";
                List_EMCY.Items.Add(errstr);
            }
        }

        private void Btn_SetHeartbeat_Click(object sender, EventArgs e)
        {
            ushort Heartbeat, Consumer;

            Heartbeat = Convert.ToUInt16(Text_Heartbeat.Text, 10);
            Consumer = Convert.ToUInt16(Text_Consumer.Text, 10);

            ret = CPM100.CPM100_NMTHeartbeat(BOARD, NODE, Heartbeat, Consumer, BlockMode);
            if (ret != 0)
            {
                errstr = "Set Heartbeat Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                errstr = "Set Heartbeat OK";
                List_EMCY.Items.Add(errstr);
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (EMCY != "")
            {
                if (List_EMCY.Items.Count >= 2000) List_EMCY.Items.Clear();

                List_EMCY.Items.Add(EMCY);
                EMCY = "";

                List_EMCY.SelectedIndex = List_EMCY.Items.Count - 1;
                List_EMCY.SelectedIndex = -1;
            }

            if (NMTErr != "")
            {
                List_EMCY.Items.Add(NMTErr);
                NMTErr = "";
            }
        }
    }
}