﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CPM100_Net;

namespace SDO_PDO_ISR
{
    public partial class Form1 : Form
    {
        byte BOARD;
        byte BAUD;
        byte NODE;

        byte BlockMode = 1;

        string RxSDO_Str = "";
        string TxSDO_Str = "";
        string RxPDO_Str = "";
        string TxPDO_Str = "";

        CPM100.ReadSDOISR RSDOISR;
        CPM100.WriteSDOISR WSDOISR;
        CPM100.RxPDOISR RXPDOISR;
        CPM100.RemotePDOISR REMOTEPDOISR;

        ushort ret;
        string errstr;

        public Form1()
        {
            InitializeComponent();
        }

        private void UserRSDOISR()
        {
            ushort Ret, Index;
            byte BoardNo, SubIndex, RLen;
            byte[] RData = new byte[4];
            uint Abort = 0;

            Ret = CPM100.CPM100_GetMasterReadSDOEvent(out BoardNo, out Index, out SubIndex);
            if (Ret == 0)
            {
                switch (Index)
                {
                    case 0x6000:	// Read DI (For CANopen SPC 401)
                        {
                            byte DI_Value = 0;

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        // DI entry number
                                        RLen = 1;
                                        RData[0] = 0x02;   // Ex: ch0~ch15, total 2-bytes (16-bits) DI data.
                                    }
                                    break;
                                case 0x01:
                                    {
                                        RLen = 1;
                                        DI_Value = 0;

                                        // Read DI value of ch0 ~ ch7
                                        //////////////////////////////////////////////
                                        //  Write user's "Read DI Data function" here


                                        //////////////////////////////////////////////

                                        RData[0] = DI_Value;
                                    }
                                    break;
                                case 0x02:
                                    {
                                        RLen = 1;
                                        DI_Value = 0;

                                        // Read DI value of ch8 ~ ch15
                                        //////////////////////////////////////////////
                                        //  Write user's "Read DI Data function" here


                                        //////////////////////////////////////////////

                                        RData[0] = DI_Value;
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        RLen = 4;
                                        RData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    case 0x6200:	// Read DO  (For CANopen SPC 401)
                        {
                            byte DO_Value = 0;

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        // DO entry number
                                        RLen = 1;
                                        RData[0] = 0x02;   // Ex: ch0~ch15, total 2-bytes (16-bits) DO data.
                                    }
                                    break;
                                case 0x01:
                                    {
                                        RLen = 1;
                                        DO_Value = 0;

                                        // Read DO value of ch0 ~ ch7
                                        //////////////////////////////////////////////
                                        //  Write user's "Read DO Data function" here


                                        //////////////////////////////////////////////

                                        RData[0] = DO_Value;
                                    }
                                    break;
                                case 0x02:
                                    {
                                        RLen = 1;
                                        DO_Value = 0;

                                        // Read DO value of ch8 ~ ch15
                                        //////////////////////////////////////////////
                                        //  Write user's "Read DO Data function" here


                                        //////////////////////////////////////////////

                                        RData[0] = DO_Value;
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        RLen = 4;
                                        RData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    default:
                        {
                            Abort = 0x06020000;

                            // Abort: Object does not exist.
                            RLen = 4;
                            RData = BitConverter.GetBytes(Abort);
                        }
                        break;
                }

                {
                    ushort i;
                    string TempStr;

                    if (Abort == 0)
                    {
                        CPM100.CPM100_ResponseMasterSDO(BoardNo, 0, Index, SubIndex, RLen, RData);
                        TempStr = "SDO Read";
                    }
                    else
                    {
                        CPM100.CPM100_SDOAbortTransmit(BoardNo, NODE, Index, SubIndex, Abort, BlockMode);
                        TempStr = "SDO Abort";
                    }

                    TempStr = TempStr + " " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":";
                    for (i = 0; i < RLen; i++)
                    {
                        TempStr = TempStr + " " + RData[i].ToString("X2");
                    }

                    TxSDO_Str = TempStr;
                }
            }
            return;
        }

        private void UserWSDOISR()
        {
            ushort Ret, Index;
            byte BoardNo, SubIndex, WLen;
            byte[] WData = new byte[4] { 0, 0, 0, 0 };
            uint Abort = 0;

            Ret = CPM100.CPM100_GetMasterWriteSDOEvent(out BoardNo, out Index, out SubIndex, out WLen, WData);
            if (Ret == 0)
            {
                switch (Index)
                {
                    case 0x6200:	// Write DO (For CANopen SPC 401)
                        {
                            if (WLen != 1)
                            {
                                Abort = 0x06070010;

                                // Abort: Data type does not match, length of service parameter does not match
                                WLen = 4;
                                WData = BitConverter.GetBytes(Abort);
                            }

                            switch (SubIndex)
                            {
                                case 0x00:
                                    {
                                        Abort = 0x06010002;

                                        // Abort: Attempt to write a read only object
                                        WLen = 4;
                                        WData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                                case 0x01:
                                    {
                                        // Ouput DO value of ch0 ~ ch7
                                        //////////////////////////////////////////////
                                        //  Write user's "Output DO Data function" here


                                        //////////////////////////////////////////////
                                    }
                                    break;
                                case 0x02:
                                    {
                                        // Change DO value of ch8 ~ ch15
                                        //////////////////////////////////////////////
                                        //  Write user's "Output DO Data function" here


                                        //////////////////////////////////////////////
                                    }
                                    break;
                                default:
                                    {
                                        Abort = 0x06090011;

                                        // Abort: Sub-index does not exist.
                                        WLen = 4;
                                        WData = BitConverter.GetBytes(Abort);
                                    }
                                    break;
                            }
                        }
                        break;
                    default:
                        {
                            Abort = 0x06090011;

                            // Abort: Object does not exist.
                            WLen = 4;
                            WData = BitConverter.GetBytes(Abort);
                        }
                        break;
                }

                {
                    ushort i;
                    string TempStr;

                    if (Abort == 0)
                    {
                        CPM100.CPM100_ResponseMasterSDO(BoardNo, 1, Index, SubIndex, 0, null);
                        TempStr = "SDO Write " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":  OK";
                    }
                    else
                    {
                        CPM100.CPM100_SDOAbortTransmit(BoardNo, NODE, Index, SubIndex, Abort, BlockMode);
                        TempStr = "SDO Abort " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":";

                        for (i = 0; i < WLen; i++)
                        {
                            TempStr = TempStr + " " + WData[i].ToString("X2");
                        }
                    }

                    RxSDO_Str = TempStr;
                }
            }
            return;
        }

        private void UserRXPDOISR()
        {
            ushort Ret;
            byte BoardNo, WLen;
            ushort CobId, i;
            byte[] WData = new byte[8] { 0, 0, 0, 0, 0, 0, 0, 0 };
            string TempStr;

            Ret = CPM100.CPM100_GetMasterRxPDOEvent(out BoardNo, out CobId, out WLen, WData);
            if (Ret == 0)
            {
                if (CobId == 0x200 + NODE)
                {
                    if (WLen >= 2)   // Ex: If user has 2-bytes DO data.
                    {
                        // Output DO value of ch0 ~ ch15
                        //////////////////////////////////////////////
                        //  Write user's "Output DO Data function" here


                        //////////////////////////////////////////////

                        TempStr = "PDO " + CobId.ToString("X3") + " Output:";
                        for (i = 0; i < WLen; i++)
                        {
                            TempStr = TempStr + " " + WData[i].ToString("X2");
                        }

                        RxPDO_Str = TempStr;
                    }
                }
            }
        }

        private void UserREMOTEPDOISR()
        {
            ushort Ret;
            byte BoardNo, RLen;
            ushort CobId, i;
            byte[] RData = new byte[8] { 0, 0, 0, 0, 0, 0, 0, 0 };
            string TempStr;

            Ret = CPM100.CPM100_GetMasterRemotePDOEvent(out BoardNo, out CobId);
            if (Ret == 0)
            {
                if (CobId == 0x180 + NODE)
                {
                    // Ex: If user has 2-bytes DI data.
                    RLen = 2;

                    // Read DI value of ch0 ~ ch15
                    //////////////////////////////////////////////
                    //  Write user's "Read DI Data function" here


                    //////////////////////////////////////////////

                    TempStr = "PDO " + CobId.ToString("X3") + " Remote:";
                    for (i = 0; i < RLen; i++)
                    {
                        TempStr = TempStr + " " + RData[i].ToString("X2");
                    }

                    TxPDO_Str = TempStr;
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            BOARD = 0;
            BAUD = 0;
            NODE = 1;

            Combo_Board.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = true;

            Combo_Board.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_Clear.Enabled = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            CPM100.CPM100_ShutdownMaster(BOARD);
        }

        private void Combo_Board_SelectedIndexChanged(object sender, EventArgs e)
        {
            BOARD = (byte)(Combo_Board.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            uint[] Entry_Data = new uint[8];

            ret = CPM100.CPM100_InitMaster(BOARD, NODE, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Board.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = false;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_Clear.Enabled = true;

                // Install SDO read ISR
                RSDOISR = new CPM100.ReadSDOISR(UserRSDOISR);
                ret = CPM100.CPM100_InstallReadSDOISR(BOARD, RSDOISR);
                if (ret != 0)
                {
                    errstr = "Install Read SDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }

                // Initial SDO write ISR
                WSDOISR = new CPM100.WriteSDOISR(UserWSDOISR);
                ret = CPM100.CPM100_InstallWriteSDOISR(BOARD, WSDOISR);
                if (ret != 0)
                {
                    errstr = "Install Write SDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }

                // Install PDO output ISR
                RXPDOISR = new CPM100.RxPDOISR(UserRXPDOISR);
                ret = CPM100.CPM100_InstallRxPDOISR(BOARD, RXPDOISR);
                if (ret != 0)
                {
                    errstr = "Install output PDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }

                // The "EntryUse" parameter and "EntryData" parameter are useless
                // when the PDO had been installed to master. 
                CPM100.CPM100_InstallPDO_List(BOARD, NODE, (ushort)(0x200 + NODE), 0, 1, (byte)255, 0, 0, Entry_Data, BlockMode);

                // Install PDO remote ISR
                REMOTEPDOISR = new CPM100.RemotePDOISR(UserREMOTEPDOISR);
                ret = CPM100.CPM100_InstallRemotePDOISR(BOARD, REMOTEPDOISR);
                if (ret != 0)
                {
                    errstr = "Install remote PDO ISR Error, = " + ret.ToString("D");
                    MessageBox.Show(errstr);
                }

                // The "EntryUse" parameter and "EntryData" parameter are useless
                // when the PDO had been installed to master.
                CPM100.CPM100_InstallPDO_List(BOARD, NODE, (ushort)(0x180 + NODE), 1, 1, (byte)255, 0, 0, Entry_Data, BlockMode);

                timer1.Enabled = true;

            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ret = CPM100.CPM100_ShutdownMaster(BOARD);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Board.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_Clear.Enabled = false;
                Combo_Node.SelectedIndex = 0;

                List_Monitor.Items.Clear();

                timer1.Enabled = false;
            }
        }

        private void Btn_Clear_Click(object sender, EventArgs e)
        {
            List_Monitor.Items.Clear();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (RxSDO_Str != "")
            {
                List_Monitor.Items.Add(RxSDO_Str);
                RxSDO_Str = "";
            }

            if (TxSDO_Str != "")
            {
                List_Monitor.Items.Add(TxSDO_Str);
                TxSDO_Str = "";
            }

            if (RxPDO_Str != "")
            {
                List_Monitor.Items.Add(RxPDO_Str);
                RxPDO_Str = "";
            }

            if (TxPDO_Str != "")
            {
                List_Monitor.Items.Add(TxPDO_Str);
                TxPDO_Str = "";
            }
        }
    }
}